###########################################################
#
# General make-include
# Johan Bezem, JB Enterprises, (c) 2008.
#
# Prerequisites:
#  - GNUmake   3.79.1 or higher
#  - Bash      2.05a or higher
#    or
#  - CMD.EXE   Windows XP or higher
#
# Version 001.000
# Saved under "001Traverse.zip"

  # Default target
  all: RECURSE

  # List_dirs will contain a space delimited list of all directories
  # containing a Makefile.
  list_dirs := $(dir $(wildcard ./*/Makefile))
  # patsubst retains only the directory names, not the full paths
  list_dirs := $(patsubst ./%,%,$(list_dirs))

  # We define RECURSE as PHONY here, so we don't need to specify it in
  # case we don't need recursion anymore (the else-clause for the following
  # ifneq, when no more subdirectories containing a Makefile can be found).
  # The directories are to be phony, in order to execute the
  # recursion commands for all directories in all cases.
  .PHONY: RECURSE $(list_dirs)

  # If list_dirs is not empty, we need to recurse through one or more 
  # subdirectories
  ifneq ($(strip $(list_dirs)),)

    # The RECURSE target can be used as a dependency for all targets
    # that need to be made recursively. Put it as the first dependency
    # for a depth-first usage.
    RECURSE: $(list_dirs)

    # To 'create' the subdirectories, we execute the commands listed,
    # one of which is reexecuting 'make' in the target subdirectory '$@'
    $(list_dirs):
	+@echo "Make[$(MAKELEVEL)]:$@"
	+@$(MAKE) --directory=$@ --no-print-directory $(MAKECMDGOALS)
    # Watch the empty line before the endif, otherwise it would be an (illegal)
    # part of the commands to $(list_dirs): !!

  endif
